package br.com.centralit.api.service.impl;

import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.ProgramaAcaoDao;
import br.com.centralit.api.model.Programa;
import br.com.centralit.api.model.ProgramaAcao;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.ProgramaAcaoService;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;

@Service("programaAcaoService")
public class ProgramaAcaoServiceImpl extends GenericServiceImpl<ProgramaAcao, Long> implements ProgramaAcaoService {

	private ProgramaAcaoDao programaAcaoDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	public ProgramaAcaoServiceImpl(ProgramaAcaoDao programaAcaoDao, @Qualifier("programaAcaoValidator") Validator validator ) {

		this.programaAcaoDao = programaAcaoDao;
		this.dao = programaAcaoDao;
		this.validator = validator;
	}

	@Override
	public boolean removeById(Long id) {
		validarExclusao(id);

		ProgramaAcao entity = getReference(id);
		
		return super.removePhysically(entity);
	}

	@Override
	public void removeList(List<ProgramaAcao> listEntity) {
		validarListaExclusao(listEntity);
		super.removeList(listEntity);
	}
	
	private void validarListaExclusao(List<ProgramaAcao> acoes) {
		for (ProgramaAcao acao : acoes) {
			validarExclusao(acao.getId());
		}
	}

	private void validarExclusao(Long id) {
		Boolean isVinculado = contratoService.existeVinculo("programaAcao.id", id);
		lancarBusinessException("VALIDACAO.REMOCAO_COM_VINCULO", CodigoErro.REGRA_NEGOCIO.getValue(), isVinculado);
	}

	@Override
	public void salvarListaAcoes(Collection<ProgramaAcao> acoes, Programa programa) {

		for(ProgramaAcao acao : acoes){

			acao.setPrograma(programa);

			validarEntidade(acao, validator);
			validarExistenciaProgramaMesmoCodigo(acao);
			validarExistenciaProgramaMesmoNome(acao);
			
			if (acao.getId() == null) {
				super.save(acao);
			} else {
				super.merge(acao);
			}
		}
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void validarExistenciaProgramaMesmoCodigo(ProgramaAcao acao) {
		
		List<ProgramaAcao> programas = (List) programaAcaoDao.buscarPorAtributo("codigo", acao.getCodigo());
		
		boolean isExistente = isExistente(acao, programas);
		
		lancarBusinessException(isExistente, "CONTRATOS.VALIDACAO.CODIGO_ACAO_DUPLICADO", CodigoErro.REGRA_NEGOCIO.getValue(), acao.getCodigo());
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void validarExistenciaProgramaMesmoNome(ProgramaAcao acao) {
		
		List<ProgramaAcao> programas = (List) programaAcaoDao.buscarPorAtributo("nome", acao.getNome());
		
		boolean isExistente = isExistente(acao, programas);
		
		lancarBusinessException(isExistente, "CONTRATOS.VALIDACAO.NOME_ACAO_DUPLICADO", CodigoErro.REGRA_NEGOCIO.getValue(), acao.getNome());
	}

	private boolean isExistente(ProgramaAcao acao, List<ProgramaAcao> acoes) {
		boolean isExistente = true;

		if (UtilColecao.isVazio(acoes)) {
			isExistente = false;
		} else {

			for (ProgramaAcao pa : acoes) {
				if (acao.getId() != null && acao.getId().equals(pa.getId())) {
					isExistente = false;
				}
			}
		}
		return isExistente;
	}

	@Override
	public Collection<ProgramaAcao> findAllByPrograma(Long programaId) {
		return this.programaAcaoDao.findAllByPrograma(programaId);
	}

}
