package br.com.centralit.api.service.impl;

import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.ProgramaDao;
import br.com.centralit.api.model.Programa;
import br.com.centralit.api.model.ProgramaAcao;
import br.com.centralit.api.service.ProgramaAcaoService;
import br.com.centralit.api.service.ProgramaService;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;

@Service("programaService")
public class ProgramaServiceImpl extends GenericServiceImpl<Programa, Long> implements ProgramaService {

	private ProgramaDao programaDao;

	@Autowired
	private ProgramaAcaoService programaAcaoService;

	@Autowired
	public ProgramaServiceImpl(ProgramaDao programaDao, @Qualifier("programaValidator") Validator validator ) {

		this.programaDao = programaDao;
		this.dao = programaDao;
		this.validator = validator;
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por trazer todos os programas da organizacao do usuário logado.
	 *
	 * @author luis.camargo
	 *
	 * @param organizacaoId ID da organizacao a se buscar os programas
	 * @return
	 */
	@Override
	public Collection<Programa> findAllByOrganizacao(Long organizacaoId) {
		return this.programaDao.findAllByOrganizacao(organizacaoId);
	}
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por trazer um programa através do id do programa ação.
	 *
	 * @author luis.camargo
	 *
	 * @param idProgramaAcao
	 * @return
	 */
	@Override
	public Programa findByProgramaAcao(Long idProgramaAcao) {
		return this.programaDao.findByProgramaAcao(idProgramaAcao);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por salvar a <code>Programa</code>
	 *
	 * @author luis.camargo
	 *
	 * @param programa
	 * @return <code>Programa</code>
	 */
	@Override
	public Programa save(Programa programa) {

		validarEntidade(programa, this.validator);
		validarExistenciaProgramaMesmoCodigo(programa);
		validarExistenciaProgramaMesmoNome(programa);

		Programa programaSaved = super.save(programa);

		programaAcaoService.salvarListaAcoes(programa.getAcoes(), programaSaved);
		
		return programaSaved;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por alterar a <code>Programa</code>
	 *
	 * @author luis.camargo
	 *
	 * @param programa
	 * @return <code>Programa</code>
	 */
	@Override
	public Programa merge(Programa programa) {

		validarEntidade(programa, this.validator);
		validarExistenciaProgramaMesmoCodigo(programa);
		validarExistenciaProgramaMesmoNome(programa);

		Programa programaMerged = super.merge(programa);

		programaAcaoService.salvarListaAcoes(programa.getAcoes(), programaMerged);

		return programa;
	}

	@Override
	public boolean removeById(Long id) {
		Programa programa = (Programa) programaDao.getReference(id);
		removerAcoes(programa.getAcoes());
		return super.removeById(id);
	}

	private void removerAcoes(Collection<ProgramaAcao> acoes) {
		programaAcaoService.removeList((List<ProgramaAcao>) acoes);
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void validarExistenciaProgramaMesmoCodigo(Programa programa) {

		List<Programa> programas = (List) programaDao.buscarPorAtributo("codigo", programa.getCodigo());
		
		boolean isExistente = isExistente(programa, programas);
		
		lancarBusinessException("CONTRATOS.VALIDACAO.CODIGO_PROGRAMA_DUPLICADO", CodigoErro.REGRA_NEGOCIO.getValue(), isExistente);
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void validarExistenciaProgramaMesmoNome(Programa programa) {
		
		List<Programa> programas = (List) programaDao.buscarPorAtributo("nome", programa.getNome());
		
		boolean isExistente = isExistente(programa, programas);
		
		lancarBusinessException("CONTRATOS.VALIDACAO.NOME_PROGRAMA_DUPLICADO", CodigoErro.REGRA_NEGOCIO.getValue(), isExistente);
	}

	private boolean isExistente(Programa programa, List<Programa> programas) {
		boolean isExistente = true;

		if (UtilColecao.isVazio(programas)) {
			isExistente = false;
		} else {

			for (Programa p : programas) {
				if (programa.getId() != null && programa.getId().equals(p.getId())) {
					isExistente = false;
				}
			}
		}
		return isExistente;
	}

	/**
	 * Método responsável por listar programas por numero ou nome e organizacao
	 *
	 * @author iago.almeida
	 *
	 * @param numeroOrNome
	 * @param idOrganizacao
	 * @return Collection<Programa>
	 */
	@Override
	public Collection<Programa> findAutoCompleteProgramaByNumeroOrNomeAndOrganizacao(String numeroOrNome, Long idOrganizacao) {

		return this.programaDao.findAutoCompleteProgramaByNumeroOrNomeAndOrganizacao(numeroOrNome, idOrganizacao);
	}

}
