package br.com.centralit.api.service.impl;

import static org.mockito.Matchers.anyLong;
import static org.mockito.Matchers.anyObject;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.security.core.context.SecurityContextHolder;

import br.com.centralit.api.dao.ProgramaAcaoDao;
import br.com.centralit.api.model.Programa;
import br.com.centralit.api.model.ProgramaAcao;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.validation.ProgramaAcaoValidator;
import br.com.centralit.framework.exception.BusinessException;
import br.com.six2six.fixturefactory.Fixture;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governan&ccedil;a Corporativa -
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="http://goo.gl/xRjvgR">607</a>
 * </p>
 *
 * @since 19/01/2016 - 15:31:20
 * @version 1.0.0
 * @author luis.camargo
 *
 */
@SuppressWarnings({ "unchecked", "rawtypes" })
@RunWith(MockitoJUnitRunner.class)
public class ProgramaAcaoServiceImplTest extends GenericServiceImplTest<ProgramaAcaoServiceImpl>{

	public static final Logger LOG = Logger.getLogger(ProgramaAcaoServiceImplTest.class);

	@InjectMocks
	private ProgramaAcaoServiceImpl service;
	
	@Mock
	private ProgramaAcaoValidator programaAcaoValidator;
	@Mock
    private ProgramaAcaoDao programaAcaoDao;
	@Mock
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;
	@Mock
	private ContratoService contratoService;

	@Override
	protected void inicializarMocksDasDependencias() {
		definirAtributo(service, "validator", programaAcaoValidator);
		definirAtributo(service, "programaAcaoDao", programaAcaoDao);
		definirAtributo(service, "configuracaoParametroSistemaService", configuracaoParametroSistemaService);
		definirAtributo(service, "contratoService", contratoService);
		
		MockitoAnnotations.initMocks(this);
	}

	@Before
	@Override
	public void setUp() {
		LOG.info("==================== Iniciando teste ProgramaAcaoServiceImplTest! ==================== ");
		SecurityContextHolder.getContext().setAuthentication(getPrincipal());
	}

	@After
	public void tearDown(){
		SecurityContextHolder.getContext().setAuthentication(null);
		LOG.info("==================== Finalizando teste ProgramaAcaoServiceImplTest! ==================== ");
	}

	@Test
	public void testRemoveSucesso() {
		ProgramaAcao programaAcao = Fixture.from(ProgramaAcao.class).gimme("valid");
		
		when(this.contratoService.existeVinculo(anyString(), anyLong())).thenReturn(false);
		when(this.programaAcaoDao.removeById(anyLong())).thenReturn(true);
		
		Boolean exclusao = service.removeById(programaAcao.getId());

		Assert.assertTrue(exclusao);
	}
	
	@Test(expected=BusinessException.class)
	public void testRemoveVinculadoContrato() {
		ProgramaAcao programaAcao = Fixture.from(ProgramaAcao.class).gimme("valid");
		
		when(this.contratoService.existeVinculo(anyString(), anyLong())).thenReturn(true);
		
		Boolean exclusao = service.removeById(programaAcao.getId());
		
		Assert.assertFalse(exclusao);
	}

	@Test
	public void testRemoveListSucesso() {
		ProgramaAcao programaAcao = Fixture.from(ProgramaAcao.class).gimme("valid");
		
		service.removeList(Arrays.asList(programaAcao));
	}
	
	@Test(expected=BusinessException.class)
	public void testRemoveListVinculadoContrato() {
		ProgramaAcao programaAcao = Fixture.from(ProgramaAcao.class).gimme("valid");

		when(this.contratoService.existeVinculo(anyString(), anyLong())).thenReturn(true);
		
		service.removeList(Arrays.asList(programaAcao));
	}

	@Test
	public void testSalvarListaAcoesSucessoNovoItem() {
		ProgramaAcao acao = Fixture.from(ProgramaAcao.class).gimme("noId");
		Programa programa = Fixture.from(Programa.class).gimme("valid");
		programa.setAcoes(Arrays.asList(acao));

		service.salvarListaAcoes(programa.getAcoes(), programa);

	}
	
	@Test
	public void testSalvarListaAcoesSucessoItemExistente() {
		Programa programa = Fixture.from(Programa.class).gimme("valid");
		
		when(this.programaAcaoDao.buscarPorAtributo(anyString(), anyObject())).thenReturn((List) programa.getAcoes());
		
		service.salvarListaAcoes(programa.getAcoes(), programa);
	}
	
	@Test(expected=BusinessException.class)
	public void testSalvarListaAcoesCodigoDuplicadoItemExistente() {
		String codigo  = "F123";

		Programa programa = Fixture.from(Programa.class).gimme("valid");
		ProgramaAcao programaAcao = Fixture.from(ProgramaAcao.class).gimme("valid");
		programaAcao.setCodigo(codigo);
		programa.setAcoes(Arrays.asList(programaAcao));

		ProgramaAcao acao = Fixture.from(ProgramaAcao.class).gimme("valid");
		acao.setId(1L);
		acao.setCodigo(codigo);
		
		when(this.programaAcaoDao.buscarPorAtributo(anyString(), anyObject())).thenReturn((List) Arrays.asList(acao));
		
		service.salvarListaAcoes(programa.getAcoes(), programa);
	}
	
	@Test(expected=BusinessException.class)
	public void testSalvarListaAcoesCodigoDuplicadoNovoItem() {
		Programa programa = Fixture.from(Programa.class).gimme("noId");
		ProgramaAcao acao = Fixture.from(ProgramaAcao.class).gimme("noId");
		programa.setAcoes(Arrays.asList(acao));
		
		ProgramaAcao acaoConsultada = Fixture.from(ProgramaAcao.class).gimme("valid");
		acao.setId(1L);
		acao.setCodigo(programa.getAcoes().iterator().next().getCodigo());
		
		when(this.programaAcaoDao.buscarPorAtributo(anyString(), anyObject())).thenReturn((List) Arrays.asList(acaoConsultada));
		
		service.salvarListaAcoes(programa.getAcoes(), programa);
	}
		
}
