package br.com.centralit.api.service.impl;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Matchers.anyObject;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.mockito.runners.MockitoJUnitRunner;

import br.com.centralit.api.dao.ProgramaDao;
import br.com.centralit.api.model.Programa;
import br.com.centralit.api.service.validation.ProgramaValidator;
import br.com.centralit.framework.exception.BusinessException;
import br.com.six2six.fixturefactory.Fixture;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governan&ccedil;a Corporativa -
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="http://goo.gl/xRjvgR">607</a>
 * </p>
 *
 * @since 14/01/2016 - 17:43:05
 * @version 1.0.0
 * @author luis.camargo
 *
 */
@SuppressWarnings({ "unchecked", "rawtypes" })
@RunWith(MockitoJUnitRunner.class)
public class ProgramaServiceImplTest extends GenericServiceImplTest<ProgramaServiceImpl>{

	public static final Logger LOG = Logger.getLogger(ProgramaServiceImplTest.class);
	
	private static final String CODIGO = "0123";
	private static final String NOME = "TESTE DO NOME DUPLICADO";


	@InjectMocks
	private ProgramaServiceImpl service;
	
	@Mock
    private ProgramaDao programaDao;
	@Mock
	private ProgramaValidator programaValidator;
	@Mock
	private ProgramaAcaoServiceImpl programaAcaoService;
	@Mock
	private ContratoServiceImpl contratoService;

	@Override
	protected void inicializarMocksDasDependencias() {
		definirAtributo(service, "programaDao", programaDao);
		definirAtributo(service, "validator", programaValidator);
		definirAtributo(service, "programaAcaoService", programaAcaoService);
		definirAtributo(service, "contratoService", contratoService);
		
		MockitoAnnotations.initMocks(this);
	}

	@Before
	public void setUp() {
		LOG.info("==================== Iniciando teste ProgramaServiceImplTest! ==================== ");
	}

	@After
	public void tearDown(){
		LOG.info("==================== Finalizando teste ProgramaServiceImplTest! ==================== ");
	}

	@Test
	public void testSaveSucesso() {
		Programa programa = Fixture.from(Programa.class).gimme("noId");
		Programa programaSaved = Fixture.from(Programa.class).gimme("valid");
		
		when(this.programaDao.save(any(Programa.class))).thenReturn(programaSaved);
		
		Programa entity = service.save(programa); 
		Assert.assertNotNull(entity);
	}
	
	
	@Test(expected = BusinessException.class)
	public void testSaveValidacaoCodigoDuplicado() {
		Programa programa = Fixture.from(Programa.class).gimme("noId");
		programa.setCodigo(CODIGO);
		
		Programa programaParaConsulta = Fixture.from(Programa.class).gimme("valid");
		programaParaConsulta.setCodigo(CODIGO);
		
		when(this.programaDao.buscarPorAtributo(anyString(), anyObject())).thenReturn((List) Arrays.asList(programaParaConsulta));
		
		Programa entity = service.save(programa); 
		
		Assert.assertNotNull(entity);
	}
	
	@Test(expected = BusinessException.class)
	public void testSaveValidacaoNomeDuplicado() {
		Programa programa = Fixture.from(Programa.class).gimme("noId");
		programa.setNome(NOME);
		
		Programa programaParaConsulta = Fixture.from(Programa.class).gimme("valid");
		programaParaConsulta.setNome(NOME);
		
		when(this.programaDao.buscarPorAtributo(anyString(), anyObject())).thenReturn((List) Arrays.asList(programaParaConsulta));
		
		Programa entity = service.save(programa); 
		Assert.assertNotNull(entity);
	}
	
	@Test
	public void testMergeSucesso() {
		Programa programa = Fixture.from(Programa.class).gimme("valid");
		Programa programaSaved = Fixture.from(Programa.class).gimme("valid");
		
		when(this.programaDao.buscarPorAtributo(anyString(), anyObject())).thenReturn((List) Arrays.asList(programa));
		when(this.programaDao.merge(any(Programa.class))).thenReturn(programaSaved);

		Programa entity = service.merge(programa);
		
		Assert.assertNotNull(entity);
	}
	
	@Test(expected = BusinessException.class)
	public void testMergeCodigoDuplicado() {
		Programa programa = Fixture.from(Programa.class).gimme("valid");
		programa.setId(1L);
		programa.setCodigo(CODIGO);
		
		Programa programaParaConsulta = Fixture.from(Programa.class).gimme("valid");
		programaParaConsulta.setId(2L);
		programaParaConsulta.setCodigo(CODIGO);
		
		when(this.programaDao.buscarPorAtributo(anyString(), anyObject())).thenReturn((List) Arrays.asList(programaParaConsulta));

		Programa entity = service.merge(programa);
		Assert.assertNotNull(entity);
	}
	
	@Test(expected = BusinessException.class)
	public void testMergeNomeDuplicado() {
		Programa programa = Fixture.from(Programa.class).gimme("valid");
		programa.setId(1L);
		programa.setNome(NOME);
		
		Programa programaParaConsulta = Fixture.from(Programa.class).gimme("valid");
		programaParaConsulta.setId(2L);
		programaParaConsulta.setNome(NOME);
		
		when(this.programaDao.buscarPorAtributo(anyString(), anyObject())).thenReturn((List) Arrays.asList(programaParaConsulta));
		
		Programa entity = service.merge(programa);
		Assert.assertNotNull(entity);
	}
	
	@Test
	public void testFindAllByOrganizacao() {
		Collection<Programa> entity = service.findAllByOrganizacao(Mockito.anyLong());
		Assert.assertNotNull(entity);
	}

	@Test
	public void testRemoveSucesso() {
		Programa programa = Fixture.from(Programa.class).gimme("valid");
		
		when(this.programaDao.getReference(anyLong())).thenReturn(programa);
		when(this.contratoService.existeVinculo(anyString(), anyLong())).thenReturn(false);
		when(this.programaDao.removeById(anyLong())).thenReturn(true);
		
		Boolean exclusao = service.removeById(programa.getId());
		
		Assert.assertTrue(exclusao);
	}
	
}
